import textwrap
from pathlib import Path
from typing import *

import toml

D = Dict[Any, Any]


def _get_config() -> D:
    """
    Return a dictionary of config values.

    First look in ~/.config/klaxon/config.toml
    Then in [tool.klaxon] in pyproject.toml
    """
    result: D = {}

    default_config_path = Path(Path.home(), ".config", "klaxon", "config.toml")

    if default_config_path.exists():
        result.update(toml.load(default_config_path))

    pyproject_toml_path = Path(Path.cwd(), "pyproject.toml")

    if pyproject_toml_path.exists():
        klaxon_options = toml.load(pyproject_toml_path)
        _recursive_update(
            result, klaxon_options.get("tool", {}).get("klaxon", {})
        )

    return result


def _recursive_update(d1: D, d2: D) -> D:
    """
    Updates d1 with values from d2 as one would expect.
    """
    for key in set(list(d1.keys()) + list(d2.keys())):
        if key in d1 and key in d2:
            if all(isinstance(d[key], dict) for d in (d1, d2)):
                _recursive_update(d1[key], d2[key])
            else:
                d1[key] = d2[key]
        elif key in d2 and key not in d1:
            d1[key] = d2[key]
    return d1


def get_notifiers_provider_config(message, subtitle, title) -> dict:
    """
    Return kwargs that will be passed to `notifiers.notify` method.
    """
    # different providers have different requirements for the `notify` method
    # most seem to take a `message` parameter, but they also have different
    # potential formatting requirements for messages.

    # use the following provider-specific map for `notify` parameters
    provider_config = {
        "pushover": {
            "message": textwrap.dedent(
                f"""
                <i>{subtitle}</i>

                {message}
                """
            ),
            "title": title,
            "html": True,
        },
        "slack": {"message": message if message else "task complete"},
    }
    return provider_config


config = _get_config()



import argparse
import logging
import os
import shlex
import subprocess as sp
import sys
from functools import partial, wraps
from typing import *

from notifiers import get_notifier

from klaxon import config
from klaxon.configuration import get_notifiers_provider_config
from klaxon.exceptions import KlaxonExit

ENABLE_PUSH_NOTIFICATIONS = config.get("enable-notifiers", False)


def klaxon(
    message="",
    title="Klaxon",
    subtitle="",
    sound="",
    push=ENABLE_PUSH_NOTIFICATIONS,
    provider_config_factory=None,
):
    """
    Wraps osascript.

    see https://apple.stackexchange.com/questions/57412/how-can-i-trigger-a-notification-center-notification-from-an-applescript-or-shel/115373#115373
    """

    escaped_message, escaped_title, escaped_subtitle, escaped_sound = [
        shlex.quote(str(e).replace(" ", "-"))
        for e in (message, title, subtitle, sound)
    ]

    if sys.platform == "darwin":
        sp.run(
            shlex.split(
                f"""osascript -e 'display notification "{escaped_message}" with title "{escaped_title}" subtitle "{escaped_subtitle}" sound name "{escaped_sound}"'"""
            )
        )
    else:
        logging.warning(
            "osascript notifications from klaxon only work on Mac OS"
        )

    if push:
        _send_push_notifications(
            message=message,
            title=title,
            subtitle=subtitle,
            provider_config_factory=provider_config_factory,
        )


def klaxonify(
    func=None,
    title="Klaxon",
    message="",
    subtitle=None,
    sound="",
    output_as_message=False,
    push=ENABLE_PUSH_NOTIFICATIONS,
    provider_config_factory=None,
):
    """
    Send a notification at the termination of a function.

    Args:
        func: the function to be decorated
        title: the notification title
        message: the notification message body
        subtitle: the notifiction subtitle
        sound: the notification sound
        output_as_message (bool): use the decorated function's output as the message body
        push: send push notification(s)
        provider_config_factory: factory for kwargs that will be passed to `notifiers.notify`

    Returns: decorated function

    """

    def decorator(function):
        @wraps(function)
        def inner(*args, **kwargs):
            result = function(*args, **kwargs)
            klaxon(
                subtitle=subtitle
                if subtitle is not None
                else function.__name__,
                message=message if not output_as_message else result,
                title=title,
                sound=sound,
                push=push,
                provider_config_factory=provider_config_factory,
            )
            return result

        return inner

    if func is not None:
        return decorator(func)
    else:
        return decorator


def _send_push_notifications(
    title,
    subtitle,
    message,
    provider_config_factory: Callable[[str, str, str], dict] = None,
):
    """Send push notifications."""
    try:
        import notifiers
    except (ImportError, ModuleNotFoundError):
        raise KlaxonExit(
            os.linesep.join(
                [
                    "notifiers enabled but not installed",
                    "$ pip(x) install klaxon[notifiers]",
                ]
            )
        )

    if "notifiers" not in config:
        raise KlaxonExit("notifiers key not found in configuration")

    message = message.strip('"').strip("'")

    provider_config = (
        get_notifiers_provider_config(message, subtitle, title)
        if provider_config_factory is None
        else provider_config_factory(message, subtitle, title)
    )

    for provider in config["notifiers"]:
        name = provider["name"]

        kwargs = {**provider_config.get(name, {}), **provider}

        if (
            "message" in get_notifier(name).required["required"]
            and "message" not in kwargs
        ):
            kwargs["message"] = message

        notifiers.notify(name, **kwargs)


def main():
    """Parse arguments from command line and pass to notify function."""
    parser = argparse.ArgumentParser(
        prog="klaxon",
        description="Send Mac OS notifications through osascript.",
    )

    parser.add_argument(
        "--message", "-m", default="", help="The body of the notification"
    )

    parser.add_argument(
        "--title", "-t", default="Klaxon", help="The notification's title"
    )

    parser.add_argument(
        "--subtitle", "-s", default="", help="The notification's subtitle"
    )

    parser.add_argument("--sound", help="The sound the notification makes")

    parser.add_argument(
        "--no-push",
        "-n",
        action="store_false",
        help="disable push notifications",
        dest="push",
        default=ENABLE_PUSH_NOTIFICATIONS,
    )

    read_stdin = sys.argv.pop() if sys.argv[-1].strip() == "--" else None

    args = parser.parse_args()

    klaxon_ = partial(
        klaxon,
        title=args.title,
        subtitle=args.subtitle,
        sound=args.sound,
        push=args.push,
    )

    if read_stdin is None:
        klaxon_(args.message)
    else:
        with sys.stdin as fd:
            klaxon_(fd.read())


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['klaxon']

package_data = \
{'': ['*']}

install_requires = \
['toml>=0.10.0,<0.11.0']

extras_require = \
{'notifiers': ['notifiers>=1.0,<2.0']}

entry_points = \
{'console_scripts': ['klaxon = klaxon:main']}

setup_kwargs = {
    'name': 'klaxon',
    'version': '0.2.9',
    'description': 'Use osascript to send notifications.',
    'long_description': '# klaxon\n\nSend Mac OS (or [notifiers][notifiers]) notifications from the terminal or Python programs.\n\nThis is especially useful for when you want a push notification \nfor some long-running background task.\n\nSimilar to the [terminal-notifier ruby gem][terminal-notifier],\nbut posix-compliant and able to send notifications via\nthe [notifiers][notifiers] library.\n\n![hello-klaxon](static/recording.gif)\n\n## Installation\nFor command-line use, the recommended method of installation is through [pipx].\n```bash\npipx install klaxon\n```\nNaturally, klaxon can also be pip-installed.\n```bash\npip install klaxon\n```\n\n## Usage\n\n### terminal\n\n```bash\n# blank notification\nklaxon\n# with custom message\nklaxon --message "this is the message body"\n# pipe message from other program\necho "this is the message body" | klaxon --\n```\n\n### python\n\n```python\nfrom klaxon import klaxon, klaxonify\n\n# send a notification\n\nklaxon(\n    title=\'hello, klaxon\',\n    subtitle=\'hola\',\n    message=\'it me\'\n)\n\n# we can decorate our functions to have\n# them send notifications at termination\n\n@klaxonify\ndef hello(name=\'world\'):\n    return f\'hello, {name}\'\n\n\n@klaxonify(title=\'oh hai\', output_as_message=True)\ndef foo():\n    return "This will be the message body."\n\n```\n\n## Non-MacOS Notifications \n\n### i.e. mobile | email | slack\n\nYou\'ll need to install klaxon with the `notifiers` extra.\n\n```bash\npipx install klaxon[notifiers]\n```\n\nYou will need a `~/.config/klaxon/config.toml` or `pyproject.toml` file with the\n`tool.klaxon` namespace configured at the top level. Values from the latter will\noverride values in the former.\n\n`enable-notifiers` will need to be set to `true` and you will need a `[[notifiers]]` key.\n\nThe latter will determine the parameters passed to the `notifiers.notify` method.\n\nFor example:\n\n`~/.config/klaxon/config.toml`\n```toml\nenable-notifiers = true\n\n[[notifiers]]\nname = \'slack\'\n# see https://api.slack.com/incoming-webhooks#getting-started\nwebhook_url = {{your webhook url}}\n\n[[notifiers]]\nname = \'pushover\'\nuser = {{your user token}}\ntoken = {{your application token}}\n```\n\nVoila! Now messages sent from klaxon will be pushed to slack and pushover.\n\n\n[terminal-notifier]: https://github.com/julienXX/terminal-notifier\n[pipx]: https://github.com/pipxproject/pipx\n[osascript]: https://apple.stackexchange.com/questions/57412/how-can-i-trigger-a-notification-center-notification-from-an-applescript-or-shel/115373#115373\n[notifiers]: https://github.com/notifiers/notifiers',
    'author': 'Stephan Fitzpatrick',
    'author_email': 'knowsuchagency@gmail.com',
    'url': 'https://pypi.org/project/klaxon/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
